<?php

declare(strict_types=1);

namespace Drupal\migrate_laminas_feed\Plugin\migrate\source;

use Drupal\migrate_laminas_feed\ReaderHttpClient;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\Plugin\migrate\source\SourcePluginBase;
use Drupal\migrate\Plugin\MigrationInterface;
use Laminas\Feed\Reader\Reader;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Migrates an RSS or Atom feed from its source URL.
 *
 * Available configuration keys:
 * - url: The source URL of the feed.
 *
 * Examples:
 *
 * @code
 * source:
 *   plugin: laminas_feed
 *   url: 'https://www.drupal.org/planet/rss.xml'
 * @endcode
 *
 * @MigrateSource(
 *   id = "laminas_feed",
 *   source_module = "migrate_laminas_feed",
 * )
 */
final class LaminasFeed extends SourcePluginBase implements ContainerFactoryPluginInterface {

  public function __construct(array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration, protected ReaderHttpClient $httpClient) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, ?MigrationInterface $migration = NULL) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('migrate_laminas_feed.reader_http_client'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __toString(): string {
    return $this->configuration['url'];
  }

  /**
   * {@inheritdoc}
   */
  protected function initializeIterator(): \ArrayIterator {
    Reader::setHttpClient($this->httpClient);
    $feed = Reader::import($this->configuration['url']);

    $items = [];
    foreach ($feed as $entry) {
      $date_created = $entry->getDateCreated();
      $date_modified = $entry->getDateModified();
      $items[] = [
        'guid' => $entry->getId(),
        'title' => $entry->getTitle(),
        'description' => $entry->getDescription(),
        'link' => $entry->getLink(),
        'permalink' => $entry->getPermalink(),
        'authors' => $entry->getAuthors(),
        // The dates are converted from \DateTime objects to timestamps for ease
        // of working with them in the process pipeline.
        'date_created' => $date_created ? $date_created->getTimestamp() : NULL,
        'date_modified' => $date_modified ? $date_modified->getTimestamp() : NULL,
        'content' => $entry->getContent(),
        'enclosure' => $entry->getEnclosure(),
        'comment_count' => $entry->getCommentCount(),
        'comment_link' => $entry->getCommentLink(),
        'comment_feed_link' => $entry->getCommentFeedLink(),
        'categories' => $entry->getCategories(),
      ];
    }
    return new \ArrayIterator($items);
  }

  /**
   * {@inheritdoc}
   */
  public function fields(): array {
    return [
      'guid' => $this->t("The item's unique ID"),
      'title' => $this->t('The item title'),
      'description' => $this->t('The item description'),
      'link' => $this->t('The URI to the HTML version of the item'),
      'permalink' => $this->t('The item permalink'),
      'authors' => $this->t('The item authors'),
      'date_created' => $this->t('The timestamp the item was created'),
      'date_modified' => $this->t('The timestamp the item was last modified'),
      'content' => $this->t('The item content'),
      'enclosure' => $this->t("The item's enclosure object"),
      'comment_count' => $this->t('The number of comments made on the item'),
      'comment_link' => $this->t('A URI where comments can be made on the item'),
      'comment_feed_link' => $this->t('A URI to a feed containing comments on the item'),
      'categories' => $this->t('The item categories'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getIds(): array {
    $ids['guid'] = [
      'type' => 'string',
    ];
    return $ids;
  }

}
