<?php

namespace Drupal\Tests\migrate_laminas_feed\Unit;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\migrate_laminas_feed\Plugin\migrate\source\LaminasFeed;
use Drupal\migrate_laminas_feed\ReaderHttpClient;
use Drupal\Tests\migrate\Unit\MigrateTestCase;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;

/**
 * @coversDefaultClass \Drupal\migrate_laminas_feed\Plugin\migrate\source\LaminasFeed
 * @group migrate_laminas_feed
 */
class LaminasFeedTest extends MigrateTestCase {

  /**
   * A mock plugin id.
   *
   * @var string
   */
  protected string $pluginId = 'placeholder_id';

  /**
   * Define bare minimum migration configuration.
   *
   * @var string[]
   */
  protected $migrationConfiguration = [
    'id' => 'LaminasFeed',
    'url' => 'https://example.com/feed',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // SourcePluginBase::prepareRow() makes a call to \Drupal::moduleHandler()
    // instead of injecting the service, so we have to mock the container to get
    // around that.
    \Drupal::unsetContainer();
    $container = new ContainerBuilder();
    $module_handler = $this->getMockBuilder(ModuleHandlerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $container->set('module_handler', $module_handler);
    \Drupal::setContainer($container);
  }

  /**
   * Builds an instance of the LaminasFeed plugin from a given HTTP response.
   *
   * @param \GuzzleHttp\Handler\MockHandler $mock_handler
   *   A Guzzle mock handler containing an HTTP response that has a feed body.
   *
   * @return \Drupal\migrate_laminas_feed\Plugin\migrate\source\LaminasFeed
   *   The migration source plugin instance.
   */
  protected function getPluginInstance(MockHandler $mock_handler): LaminasFeed {
    $handler = HandlerStack::create($mock_handler);
    $client = new Client(['handler' => $handler]);
    $reader_http_client = new ReaderHttpClient($client);
    return new LaminasFeed($this->migrationConfiguration, $this->pluginId, [], $this->getMigration(), $reader_http_client);
  }

  /**
   * Tests parsing of Atom feeds.
   */
  public function testAtomFeed(): void {
    $mock = new MockHandler([
      new Response(
        200,
        ['Content-Type' => 'application/atom+xml'],
        file_get_contents(__DIR__ . '/../../fixtures/atom_sample.xml'),
      ),
    ]);
    $plugin = $this->getPluginInstance($mock);

    $this->assertEquals(2, $plugin->count());

    $plugin->next();
    $row1 = $plugin->current()->getSource();
    $this->assertEquals('Atom-Powered Robots Run Amok', $row1['title']);
    $this->assertEquals('http://example.org/2003/12/13/atom03', $row1['link']);
    $this->assertEquals('Some text.', $row1['description']);
    $this->assertEquals('urn:uuid:1225c695-cfb8-4ebb-aaaa-80da344efa6a', $row1['guid']);
    $this->assertEquals(1071340202, $row1['date_modified']);

    $plugin->next();
    $row2 = $plugin->current()->getSource();
    $this->assertEquals('We tried to stop them, but we failed.', $row2['title']);
    $this->assertEquals('http://example.org/2003/12/14/atom03', $row2['link']);
    $this->assertEquals('Some other text.', $row2['description']);
    $this->assertEquals('urn:uuid:1225c695-cfb8-4ebb-bbbb-80da344efa6a', $row2['guid']);
    $this->assertEquals(1071419402, $row2['date_modified']);
  }

  /**
   * Tests parsing of RSS v0.91 feeds.
   */
  public function testRss091Feed(): void {
    $mock = new MockHandler([
      new Response(
        200,
        ['Content-Type' => 'application/rss+xml'],
        file_get_contents(__DIR__ . '/../../fixtures/rss091_sample.xml'),
      ),
    ]);
    $plugin = $this->getPluginInstance($mock);

    $this->assertEquals(2, $plugin->count());

    $plugin->next();
    $row1 = $plugin->current()->getSource();
    $this->assertEquals('First example feed item title', $row1['title']);
    $this->assertEquals('http://example.com/example-turns-one', $row1['link']);
    $this->assertEquals('First example feed item description.', $row1['description']);

    $plugin->next();
    $row2 = $plugin->current()->getSource();
    $this->assertEquals('Second example feed item title.', $row2['title']);
    $this->assertEquals('http://example.com/example-turns-two', $row2['link']);
    $this->assertEquals('Second example feed item description.', $row2['description']);
  }

}
